# Generated by Django 4.1.2 on 2022-10-31 13:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcaps', '0002_pcap_rule_to_validate'),
        ('snort', '0002_alter_snortrule_options_alter_snortrule_extra_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='snortrule',
            name='pcap_validation',
            field=models.ManyToManyField(blank=True, null=True, related_name='validation_pcap', to='pcaps.pcap'),
        ),
    ]



from django.contrib import admin
from django import forms
import os
from .models import Pcap

@admin.register(Pcap)
class SnortRuleAdmin( admin.ModelAdmin):
    def validate(self, request, obj:Pcap):
        # todo test saved rule vs pcap
        print("validate button pushed", obj.name)
    validate.label = "validate"  # optional
    validate.color = "green"
    validate.short_description = "Submit this article"  # optional


    change_actions = ('validate', )
    changelist_actions = ('validate',)

    list_display = ("name", "description", "pcap_file", "rule_to_validate", "date")
    search_fields = ("name", "description", "pcap_file", "rule_to_validate")
    # form = SnortRuleAdminForm




# Generated by Django 4.1.2 on 2022-10-31 13:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcaps', '0002_pcap_rule_to_validate'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pcap',
            name='extra',
            field=models.TextField(blank=True, max_length=256),
        ),
    ]



"""
Django settings for snort_web_master project.

Generated by 'django-admin startproject' using Django 4.1.2.

For more information on this file, see
https://docs.djangoproject.com/en/4.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.1/ref/settings/
"""

from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-6imn8m6=okyge)ghoygg&rz3zzei45f5!5%#*mmy1e*ekj*cq5'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'snort',
    'pcaps',
    "django_object_actions",

]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'snort_web_master.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'snort_web_master.wsgi.application'


# Database
# https://docs.djangoproject.com/en/4.1/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}


# Password validation
# https://docs.djangoproject.com/en/4.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.1/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.1/howto/static-files/

STATIC_URL = 'static/'

# Default primary key field type
# https://docs.djangoproject.com/en/4.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'
# LDAP_SERVERS = [
#     {
#         'host': 'ldap.forumsys.com',
#         'port': 389,
#         'use_ssl': False,
#         'get_info': 'NONE',
#     }
# ]
# # LDAP_ENGINE = 'OpenLDAP'
# # https://coderbook.com/@marcus/how-to-add-ldap-and-active-directory-authentication-to-django/
# LDAP_BIND_USER = "cn=read-only-admin,dc=example,dc=com"
# LDAP_BIND_PASSWORD = "password"
# from ldap3 import NTLM
# LDAP_AUTHENTICATION = NTLM
# LDAP_SEARCH_BASE = "dc=example,dc=com"
# LDAP_USER_SEARCH_FILTER = "(&(sAMAccountName=%s)(objectClass=user))"
# LDAP_ATTRIBUTES_MAP = {
#     'username': 'sAMAccountName',
#     'first_name': 'givenName',
#     'last_name': 'sn',
#     'email': 'mail',
# }
# LDAP_GROUPS_MAP = {
#     'mathematicians': "cn=read-only-admin,dc=domain,dc=com",
# }
# LDAP_MIN_GROUPS = ["mathematicians", ]
# AUTHENTICATION_BACKENDS = ("django_auth_ldap3_ad.auth.LDAP3ADBackend",)
# from django_auth_ldap3_ad.ad_users import Aduser
#
# adu = Aduser()
# adu.create_ad_user(user_dn, firstname, lastname, sAMAccountName, mail=email, description=description)
# adu.update_password_ad_user(user_dn, password)
# adu.update_ad_user(user_dn, {'ldapAttrib1': 'value1', 'ldapAttrib2': 'value2'})
# adu.activate_ad_user(user_dn)


# Generated by Django 4.1.2 on 2022-10-31 09:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='snortrule',
            options={'ordering': ('name', 'type', 'date')},
        ),
        migrations.AlterField(
            model_name='snortrule',
            name='extra',
            field=models.TextField(blank=True, max_length=256),
        ),
        migrations.AlterField(
            model_name='snortrule',
            name='group',
            field=models.TextField(blank=True, max_length=100),
        ),
        migrations.AlterField(
            model_name='snortrule',
            name='main_ref',
            field=models.TextField(blank=True, max_length=12),
        ),
        migrations.AlterField(
            model_name='snortrule',
            name='request_ref',
            field=models.TextField(blank=True, max_length=12),
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-07 06:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0007_remove_snortrule_template'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snortrule',
            name='group',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AlterField(
            model_name='snortrule',
            name='location',
            field=models.CharField(max_length=256),
        ),
        migrations.AlterField(
            model_name='snortrule',
            name='main_ref',
            field=models.CharField(blank=True, max_length=12),
        ),
        migrations.AlterField(
            model_name='snortrule',
            name='request_ref',
            field=models.CharField(blank=True, max_length=12),
        ),
    ]



from django.db import models
from pcaps.models import Pcap
from .snort_templates import types_list

class SnortRule(models.Model):
    group = models.CharField(max_length=100, blank=True)
    user = models.CharField(max_length=100, blank=True, default=0)
    name = models.CharField(max_length=100)
    type = models.TextField(max_length=30, choices=types_list)
    content = models.TextField(max_length=2048)
    description = models.TextField(max_length=256)
    extra = models.TextField(max_length=256, blank=True)
    location = models.CharField(max_length=256)
    request_ref = models.CharField(max_length=12, blank=True)
    main_ref = models.CharField(max_length=12, blank=True)
    date = models.DateTimeField(auto_now=True)
    pcap_validation = models.ManyToManyField(Pcap, related_name='validation_pcap', blank=True)
    objects = models.Manager()

    class Meta:
        ordering = ("name", "type", "date")





import os
import time

from django import forms
from .models import SnortRule
from django.utils.html import format_html
from urllib.parse import quote as urlquote
from .snort_templates import snort_type_to_template, types_list, EMPTY_TYPES
from .parser import Parser
from django.contrib import messages
from django.utils.translation import gettext
# Register your models here.
from django.contrib import admin
from django_object_actions import DjangoObjectActions
import subprocess

# todo: active directory sync users
# todo: fix the sig structure

class SnortRuleAdminForm(forms.ModelForm):
    class Meta:
        model = SnortRule
        fields = "__all__"

    def clean_type(self):
        if not dict(types_list).get(self.cleaned_data.get("type")):
            raise forms.ValidationError("cant find type, did you forgot it? or forgot to add type to db")
        return self.cleaned_data.get("type")

    def clean_content(self):
        if not (self.cleaned_data.get("type")):
            return self.clean_type()
        if self.cleaned_data.get("type") in EMPTY_TYPES:
            if self.cleaned_data["content"] != "":
                raise forms.ValidationError("rule cannot contain content in the current chosen type")
        else:
            if self.cleaned_data["content"] == "":
                raise forms.ValidationError("rule cannot be empty in the current chosen type")
            else:
                try:
                    rule_template = snort_type_to_template[dict(types_list)[self.cleaned_data.get("type")]]().get_rule("Test",
                                                                                                   sig_name="Test",
                                                                                                   sig_content=self.cleaned_data["content"],
                                                                                                   writer_team="Test",
                                                                                                   sig_writer="test",
                                                                                                   main_doc="0",
                                                                                                   cur_date=time.time(),
                                                                                                   sig_ref="0",
                                                                                                   sig_desc="Test")
                    parser = Parser(rule_template)
                    parser.parse_header()
                    parser.parse_options()
                except Exception as e:
                    raise forms.ValidationError(e)

        return self.cleaned_data["content"]

    def clean_location(self):
        try:
            if os.path.dirname(self.cleaned_data["location"]) != "":
                os.makedirs(os.path.dirname(self.cleaned_data["location"]), exist_ok=True)
            os.makedirs(os.path.dirname(self.cleaned_data["location"]), exist_ok=True)
            with open(self.cleaned_data["location"], "w") as rule_file:
                rule_file.write(self.cleaned_data["content"])
        except Exception as e:
            forms.ValidationError(e)
        return self.cleaned_data["location"]

    def clean_pcap_validation(self):
        if not self.cleaned_data.get("pcap_validation"):
            raise forms.ValidationError("pcap validateion cannot be empty, please find a pcap that aggre with your rule")
        cur_rule = SnortRule()
        cur_rule.content = self.data.get("content")
        cur_rule.location = self.data.get("location")
        cur_rule.group = self.data.get("group")
        cur_rule.id = self.data.get("id")
        cur_rule.main_ref = self.data.get("main_ref")
        cur_rule.name = self.data.get("name")
        cur_rule.type = self.data.get("type")
        cur_rule.user = self.data.get("user")
        cur_rule.request_ref = self.data.get("request_ref")

        validate_pcap_snort(self.cleaned_data.get("pcap_validation"), cur_rule)
        return self.cleaned_data["pcap_validation"]


def validate_pcap_snort(pcaps, rule):
    rule_template = snort_type_to_template[dict(types_list)[rule.type]]().get_rule(rule.group, sig_name=rule.name, sig_content=rule.content, writer_team=rule.group, sig_writer=rule.user, main_doc=rule.main_ref, cur_date=time.time(), sig_ref=rule.request_ref, sig_desc=rule.description)
    if not rule.location:
        import re, unicodedata
        rule.location = re.sub(r'[-\s]+', '-',re.sub(r'[^\w\s-]', '',
                                 rule.name)
                          .strip()
                          .lower())

    with open(rule.location + ".tmp", "w") as rule_file:
        rule_file.write(rule_template)
    for pcap in pcaps:
        try:
            if not verify_leagal_pcap("/app/{pcap.pcap_file}"):
                raise Exception(f"illegal pcap file")
            if not os.path.exists(f"/app/{pcap.pcap_file}"):
                raise Exception(f"cant find file /app/{pcap.pcap_file}")
            stdout, stderr = subprocess.Popen(["/home/snorty/snort3/bin/snort", "-R", rule.location + ".tmp", "-r", f"/app/{pcap.pcap_file}", "-A", "fast"], stdout=subprocess.PIPE,
                                              stderr=subprocess.PIPE).communicate()
            if stdout and not stderr:
                if "total_alerts: " in stdout:
                    return "alert matched"
                else:
                    stderr = "alert did not matched rule"
            if stderr:
                raise forms.ValidationError(stderr + "<br>----------" + stdout.decode())
        except Exception as e:
            raise forms.ValidationError(f"could not validate rule on {pcap.pcap_file}: {e}")
        return stdout


def verify_leagal_pcap(filename):
    import dpkt
    counter = 0

    for ts, pkt in dpkt.pcap.Reader(open(filename, 'br')):

        counter += 1
        eth = dpkt.ethernet.Ethernet(pkt)
        if eth.type != dpkt.ethernet.ETH_TYPE_IP:
            continue

    if not counter:
        return False
    return True

@admin.register(SnortRule)
class SnortRuleAdmin(DjangoObjectActions, admin.ModelAdmin):
    change_actions = ('validate',)
    changelist_actions = ('validate',)

    list_display = ("name", "type", "description", "date", "main_ref")
    search_fields = ("name", "description", "content", "template", "type", "main_ref", "request_ref")
    form = SnortRuleAdminForm


    def validate(self, request, obj:SnortRule):
        # todo test saved rule vs pcap
        error = ""
        stdout = ""
        status = messages.ERROR
        try:
            stdout = validate_pcap_snort(obj.pcap_validation.all(), obj)
            stdout = stdout.decode().replace('\n', '<br>')
            status = messages.SUCCESS
            message = "The {name} “{obj}” run validation on {pcap} with results:<br>"+ f"{stdout}"
        except Exception as e:
            error = e
        if os.name =="nt":
            error = f"cannot validate on windows"
        if error:
            message = "The {name} “{obj}” failed validation on {pcap} due to ERROR:<b>{err}</b>"
        msg_dict = {
            "name": obj._meta.verbose_name,
            "obj": format_html('<a href="{}">{}</a>', urlquote(request.path), obj),
            "pcap": "all",
            "err": error
        }
        msg = format_html(
            gettext(message), **msg_dict
        )
        self.message_user(request, msg, status)
        return self.change_view(request, str(obj.id))
    validate.label = "validate on pcaps"  # optional
    validate.color = "green"
    readonly_fields = ('location', "user")
    validate.short_description = "test rule on pcaps"  # optional






# Generated by Django 4.1.2 on 2022-11-07 06:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0004_alter_snortrule_pcap_validation'),
    ]

    operations = [
        migrations.AlterField(
            model_name='snortrule',
            name='template',
            field=models.TextField(choices=[('abcdefghijklmnop', 'type a')], max_length=30),
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-07 11:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0008_alter_snortrule_group_alter_snortrule_location_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='snortrule',
            name='user',
            field=models.CharField(blank=True, default=0, max_length=100),
        ),
    ]



# Generated by Django 4.1.2 on 2022-10-31 13:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcaps', '0003_alter_pcap_extra'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='pcap',
            name='location',
        ),
        migrations.AddField(
            model_name='pcap',
            name='pcap_file',
            field=models.FileField(null=True, upload_to='pacp_repo/'),
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-07 06:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0006_alter_snortrule_template_alter_snortrule_type'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='snortrule',
            name='template',
        ),
    ]
